/// dialog.scss -*- Mode: scss; indent-tabs-mode: nil; basic-offset: 2 -*-
@import 'extends';

@mixin fullscreen {
  max-width: 100%;
  max-height: 100%;
  width: 100%;
  height: 100%;
  border-radius: 0;
  flex-basis: 100%;
}

body.popup {
  md-dialog {
    &[flex-xs="100"],
    &.flex-xs-100 {
      @include fullscreen;
    }
  }
}

md-dialog {
  background-color: white !important;
  md-dialog-content {
    &.sg-dialog-message {
      // Prepare content to receive a "close" button on the right
      align-items: flex-start;
      display: flex;
      flex-shrink: 0;
      transition: background-color $swift-ease-in-duration $swift-ease-in-timing-function;
      &.ng-hide {
        background-color: white !important;
        transition-duration: 0s;
      }
    }
  }
}

@media (max-width: $layout-breakpoint-xs) {
  md-dialog {
    &[flex-xs="100"],
    &.flex-xs-100 {
      @include fullscreen;
    }
  }
}

