/// menu.scss -*- Mode: scss; indent-tabs-mode: nil; basic-offset: 2 -*-
@import 'extends';

md-card {
  md-card-actions:not(.layout-column) {
    .md-menu {
      padding: 0; // don't add padding so all icon buttons are of the same height
    }
  }
}

md-sidenav {
  md-list {
    .md-menu {
      padding: 0; // don't add padding so all icon buttons are of the same height
      opacity: 1;
      transition: $swift-ease-in;
      &.ng-hide {
        opacity: 0;
      }
    }
  }
}


// submenu hack
// See https://github.com/angular/material/issues/6096
md-menu-content {
  .md-menu {
    @extend md-menu-item;
    padding: 0;
    > .md-button {
      line-height: $menu-item-height;
      &:after {
        display: block;
        content: '\25BC';
        position: absolute;
        top: 0px;
        speak: none;
        transform: rotate(270deg) scaleY(0.45) scaleX(0.9);
        right: 2 * $baseline-grid;
      }
    }
  }
}
