/*
  Copyright (C) 2005-2017 Inverse inc.  
  Copyright (C) 2004-2005 SKYRIX Software AG

  This file is part of SOGo.

  SOGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  SOGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along withS OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/

#ifndef __Mailer_UIxMailRenderingContext_H__
#define __Mailer_UIxMailRenderingContext_H__


#include <NGObjWeb/WOContext.h>


/*
  UIxMailRenderingContext
  
  The rendering context is used to track nesting of mail part viewers.
*/

@class NSArray;
@class NSData;
@class NSDictionary;
@class WOComponent;

@interface UIxMailRenderingContext : NSObject
{
  WOComponent  *viewer;       /* non-retained! */
  WOContext    *context;      /* non-retained! */
  NSDictionary *flatContents; /* IMAP4 name to NSData */
}

- (id) initWithViewer: (WOComponent *) _viewer
	      context: (WOContext *) _ctx;

/* state */

- (void) reset;

/* fetching */

- (NSDictionary *) flatContents;
- (NSData *) flatContentForPartPath: (NSArray *) _partPath;

/* viewer components */

- (WOComponent *) viewerForBodyInfo: (id)_info;

@end

@interface WOContext(UIxMailPart)

- (void) pushMailRenderingContext: (UIxMailRenderingContext *) _mctx;
- (UIxMailRenderingContext *) popMailRenderingContext;
- (UIxMailRenderingContext *) mailRenderingContext;

@end

#endif /* __Mailer_UIxMailRenderingContext_H__ */
