
/** Theme styles for sgTimepicker. */
sg-timepicker.md-THEME_NAME-theme {
  background: white; }

.md-THEME_NAME-theme {
  .sg-timepicker-input-container {
    border-bottom-color: '{{background-300}}'; }
  .sg-timepicker-input-container.sg-timepicker-focused {
    border-bottom-color: '{{primary-500}}'; }
  .sg-timepicker-input-container.sg-timepicker-invalid {
    border-bottom-color: '{{warn-500}}'; }
  .sg-timepicker-time-pane {
  border-color: '{{background-300}}'; }
  .sg-timepicker-triangle-button:hover .sg-timepicker-expand-triangle {
  border-top-color: '{{foreground-2}}'; }
  .sg-timepicker-open .sg-timepicker-time-icon {
  fill: '{{primary-500}}'; }
  .sg-timepicker-calendar {
  background: white; }
}