/// angular.scss -*- Mode: scss; indent-tabs-mode: nil; basic-offset: 2 -*-

// ngCloak directive
[ng\:cloak], [ng-cloak], [data-ng-cloak], [x-ng-cloak], .ng-cloak, .x-ng-cloak {
  display: none !important;
}

.sg-hide {
  opacity: 0;
}
