/// hotkeys.scss -*- Mode: scss; indent-tabs-mode: nil; basic-offset: 2 -*-

.sg-hotkey-container {
  display: inline;
  text-align: right;
  min-width: 7em;
}

sg-hotkey {
  background-color: #333;
  border-radius: 5px;
  border: 1px solid #333;
  box-shadow: inset 0 1px 0 #666, 0 1px 0 #bbb;
  color: #fff;
  display: inline-block;
  padding: 5px 9px;
  text-align: center;
}
