/* NGVCardReference.m - this file is part of NGCards
 *
 * Copyright (C) 2008-2019 Inverse inc.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#import <Foundation/NSString.h>

#import "NGVCardReference.h"

@implementation NGVCardReference

- (void) setFn: (NSString *) newFn
{
  [self setValue: 0 ofAttribute: @"fn" to: newFn];
}

- (NSString *) fn
{
  return [self value: 0 ofAttribute: @"fn"];
}

- (void) setEmail: (NSString *) newEmail
{
  [self setValue: 0 ofAttribute: @"email" to: newEmail];
}

- (NSString *) email
{
  return [self value: 0 ofAttribute: @"email"];
}

- (void) setReference: (NSString *) newReference
{
  [self setSingleValue: newReference forKey: @""];
}

- (NSString *) reference
{
  return [self flattenedValuesForKey: @""];
}

@end
