/// datePicker.scss -*- Mode: scss; indent-tabs-mode: nil; basic-offset: 2 -*-
@import 'extends-datePicker';

.md-calendar-scroll-container,
.md-datepicker-input-mask {
  // Let the content set the container width instead of using a fixed width
  width: auto;
}

.sg-datepicker-readonly-input-container {
  border-bottom: 0;
}