/// content.scss -*- Mode: scss; indent-tabs-mode: nil; basic-offset: 2 -*-
@import 'extends';

// angular-material is mobile first (is it ?) SOGo is not. We're inversing
// the media definitions and replacing magic numbers with variables
// ----------------------------------------------------------------------------
//md-content {
//  &.md-padding {
//    padding: $mg;
//  }
//}
// Overrides out of specs angular-material definition
//@include at(sm) {
//  md-content.md-padding {
//    padding: $mg;
//  }
//}

.sg-subcontent {
  border-left: $baseline-grid solid sg-color($sogoGreen, 100);
  margin-left: ($baseline-grid * 0.5);
  padding-left: $baseline-grid;
}

.sg-inset {
  // Inspired from <md-divider md-inset>
  margin-left: $baseline-grid * 9;
}