/* UIxMailUserRightsEditor.h - this file is part of SOGo
 *
 * Copyright (C) 2007-2013 Inverse inc.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef UIXMAILUSERRIGHTSEDITOR_H
#define UIXMAILUSERRIGHTSEDITOR_H

#import <UI/Common/UIxUserRightsEditor.h>

@interface UIxMailUserRightsEditor : UIxUserRightsEditor

- (void) setUserCanReadMails: (BOOL) userCanReadMails;
- (BOOL) userCanReadMails;

- (void) setUserCanWriteMails: (BOOL) userCanWriteMails;
- (BOOL) userCanWriteMails;

- (void) setUserCanInsertMails: (BOOL) userCanInsertMails;
- (BOOL) userCanInsertMails;

- (void) setUserCanMarkMailsRead: (BOOL) userCanMarkMailsRead;
- (BOOL) userCanMarkMailsRead;

- (void) setUserCanEraseMails: (BOOL) userCanEraseMails;
- (BOOL) userCanEraseMails;

- (void) setUserCanCreateSubfolders: (BOOL) userCanCreateSubfolders;
- (BOOL) userCanCreateSubfolders;

- (void) setUserCanPostMails: (BOOL) userCanPostMails;
- (BOOL) userCanPostMails;

- (void) setUserIsAdministrator: (BOOL) userIsAdministrator;
- (BOOL) userIsAdministrator;

@end

#endif /* UIXMAILUSERRIGHTSEDITOR_H */
