/// styles.scss -*- Mode: scss; indent-tabs-mode: nil; basic-offset: 2 -*-

@use "sass:math";

// Breakpoint (https://github.com/at-import/breakpoint)
// ------------------------------------------------------------------------------
@import 'breakpoint';

@import 'core/variables'; // Will be replace by a complete 'config file

// Browser reset - most effective for applications. Generated by Compass in a non-regular way
// ------------------------------------------------------------------------------
//@import 'core/reset';

// CORE elements
// ------------------------------------------------------------------------------
@import 'core/functions';
@import 'core/angular';

// Original angular-material core style and services
// ------------------------------------------------------------------------------
@import 'core/mixins';
@import 'core/layout-attributes';
@import 'core/structure';
@import 'core/layout';
@import 'core/typography';

// Original angular-material components
// ------------------------------------------------------------------------------
@import 'components/input/input-variables';
@import 'components/autocomplete/autocomplete';
@import 'components/backdrop/backdrop';
//@import 'components/bottomSheet/bottom-sheet';
@import 'components/button/button';
@import 'components/card/card';
@import 'components/checkbox/checkbox';
@import 'components/chips/chips';
@import 'components/content/content';
@import 'components/datepicker/calendar';
@import 'components/datepicker/datePicker';
@import 'components/dialog/dialog';
@import 'components/divider/divider';
//@import 'components/fabActions/fabActions';
@import 'components/fabSpeedDial/fabSpeedDial';
//@import 'components/fabToolbar/fabToolbar';
@import 'components/gridList/grid-list';
@import 'components/icon/icon';
@import 'components/input/input';
@import 'components/list/list';
@import 'components/menu/menu';
@import 'components/panel/panel';
@import 'components/progressCircular/progress-circular';
@import 'components/progressLinear/progress-linear';
@import 'components/radioButton/radio-button';
@import 'components/select/select';
@import 'components/sidenav/sidenav';
@import 'components/slider/slider';
@import 'components/sticky/sticky';
@import 'components/subheader/subheader';
//@import 'components/swipe/swipe';
@import 'components/switch/switch';
@import 'components/tabs/tabs';
@import 'components/toast/toast';
@import 'components/toolbar/toolbar';
@import 'components/tooltip/tooltip';
@import 'components/truncate/truncate';
@import 'components/virtualRepeat/virtual-repeat';
@import 'components/whiteframe/whiteframe';

// Inverse components
@import 'components/avatarImage/avatarImage';
@import 'components/colorpicker/colorpicker';
@import 'components/draggable-droppable/draggable';
@import 'components/draggable-droppable/droppable';
@import 'components/hotkeys/hotkeys';
@import 'components/imagegallery/imagegallery';
@import 'components/ripple/ripple';
@import 'components/timepicker/timepicker';
@import 'components/pseudo-input/pseudo-input';
@import 'components/block-toggle/block-toggle';
@import 'views/view';
@import 'core/print';

// dev-tools - NOT FOR PRODUCTION
// ------------------------------------------------------------------------------
//@import 'devtools';
