/// PreferencesUI.scss -*- Mode: scss; indent-tabs-mode: nil; basic-offset: 2 -*-

[ui-view="signature"] {
  .text-preview {
    white-space: pre-wrap;
    word-wrap: break-word;
  }

  .sub-information {
    margin-bottom: 14px;
    text-align: left;
    font-size: 12px;
    line-height: 14px;
    font-weight: 600;
  }

  // CKEditor
  .ck-editor__editable {
    min-height: 70px;
  }

  .ck-widget.raw-html-embed .raw-html-embed__preview-content>* {
      margin-left: unset!important;
      margin-right: unset!important;
  }
  
  .ck-sticky-panel__content {
    border: 0!important;
  }
}